@@ -20,6 +20,7 @@ from .virtual_machine import MEM_SIZE
 
 TOKENS = 0
 CODE = 1
+LINE_NUMBER = 2
 
 DONT_INIT = "?"
 
@@ -47,7 +48,7 @@ def add_debug(s, vm):
     vm.debug += (s + "\n")
 
 
-def minus_token(token_line, pos):
+def minus_token(token_line, pos, line_num):
     """
     Negates the next token if minus token found
 
@@ -59,12 +60,12 @@ def minus_token(token_line, pos):
         if isinstance(token_line[pos + 1], IntegerTok):
             token_line[pos + 1].negate_val()
         else:
-            raise InvalidArgument("-")
+            raise InvalidArgument("-", line_num)
     except Exception:
-        raise InvalidArgument("-")
+        raise InvalidArgument("-", line_num)
 
 
-def register_token(token_line, pos, vm):
+def register_token(token_line, pos, vm, line_num):
     if vm.flavor == "intel" or vm.flavor == "att":
 
         return (token_line[pos], pos + 1)
@@ -72,14 +73,14 @@ def register_token(token_line, pos, vm):
           isinstance(token_line[pos + 1], OpenParen)):
         reg = None
         disp = None
-        reg, disp, pos = get_address_mips(token_line, pos + 2, vm,
+        reg, disp, pos = get_address_mips(token_line, pos + 2, vm, line_num,
                                           token_line[pos])
         return (RegAddress(reg.get_nm(), vm, disp), pos)
     else:
         return (token_line[pos], pos + 1)
 
 
-def number_token(token_line, pos, vm):
+def number_token(token_line, pos, vm, line_num):
     """
     If token seen is an integer, determine by flavor whether
     to return the integer token or return an address token
@@ -100,24 +101,27 @@ def number_token(token_line, pos, vm):
         reg = None
         disp = None
         if vm.flavor == "att":
-            reg, disp, pos = get_address_att(token_line, pos + 2, vm,
-                                             token_line[pos].get_val())
+            reg, disp, pos = get_address_att(token_line, pos + 2,
+                                             vm, line_num,
+                                             token_line[pos].get_val(line_num))
         else:
-            reg, disp, pos = get_address_mips(token_line, pos + 2, vm,
-                                              token_line[pos].get_val())
+            reg, disp, pos = get_address_mips(
+                token_line, pos + 2,
+                vm, line_num,
+                token_line[pos].get_val(line_num))
         if reg:
             return (RegAddress(reg.get_nm(), vm, disp,
                                reg.get_multiplier()), pos)
         else:
             return (Address(hex(disp).split('x')[-1].upper(), vm), pos)
     elif vm.flavor == "att" and token_line[pos].con is False:
-        mem_val = token_line[pos].get_val()
+        mem_val = token_line[pos].get_val(line_num)
         return (Address(hex(mem_val).split('x')[-1].upper(), vm), pos + 1)
     else:
         return (token_line[pos], pos + 1)
 
 
-def symbol_token(token_line, pos, vm):
+def symbol_token(token_line, pos, vm, line_num):
     """
     If token seen is a symbol, determine by flavor whether
     to return the symbol token or return an address token
@@ -134,7 +138,7 @@ def symbol_token(token_line, pos, vm):
         return (Symbol(token_line[pos].get_nm(), vm), pos + 1)
     elif (pos + 1 < len(token_line) and
           isinstance(token_line[pos + 1], OpenParen)):
-        reg, disp, pos = get_address_mips(token_line, pos + 2, vm,
+        reg, disp, pos = get_address_mips(token_line, pos + 2, vm, line_num,
                                           vm.symbols[token_line[pos].get_nm()])
         return (RegAddress(reg.get_nm(), vm, disp,
                            reg.get_multiplier()), pos)
@@ -165,7 +169,7 @@ def is_start_address(token_line, pos, flavor):
         return False
 
 
-def get_data_type(token_line, pos):
+def get_data_type(token_line, pos, line_num):
     """
     Returns the data type
 
@@ -177,17 +181,17 @@ def get_data_type(token_line, pos):
         Data type of variable
     """
     if pos >= len(token_line):
-        raise MissingData()
+        raise MissingData(line_num)
     elif isinstance(token_line[pos], DataType):
         if token_line[pos].get_nm() not in dtype_info:
-            raise InvalidDataType(token_line[pos].get_nm())
+            raise InvalidDataType(token_line[pos].get_nm(), line_num)
         else:
             return token_line[pos].get_nm()
     else:
-        raise InvalidDataType(token_line[pos].get_nm())
+        raise InvalidDataType(token_line[pos].get_nm(), line_num)
 
 
-def get_data_token(token_line, pos):
+def get_data_token(token_line, pos, line_num):
     """
     Returns the data token found
 
@@ -199,33 +203,35 @@ def get_data_token(token_line, pos):
         Data value, next position
     """
     if pos >= len(token_line):
-        raise MissingData()
+        raise MissingData(line_num)
     if isinstance(token_line[pos], MinusTok):
         try:
             if isinstance(token_line[pos + 1], IntegerTok):
                 token_line[pos + 1].negate_val()
-                return get_data_token(token_line, pos + 1)
+                return get_data_token(token_line, pos + 1, line_num)
             else:
-                raise InvalidDataVal("-")
+                raise InvalidDataVal("-", line_num)
         except Exception:
-            raise InvalidDataVal("-")
+            raise InvalidDataVal("-", line_num)
     elif isinstance(token_line[pos], IntegerTok):
-        return token_line[pos].get_val(), pos + 1
+        return token_line[pos].get_val(line_num), pos + 1
     elif isinstance(token_line[pos], FloatTok):
         if (token_line[pos].get_type() == ".float" and
-                token_line[pos].get_val() > float(2 ** 22)):
-            raise TooBigForSingle(str(token_line[pos].get_val()))
+                token_line[pos].get_val(line_num) > float(2 ** 22)):
+            raise TooBigForSingle(str(token_line[pos].get_val(line_num)),
+                                  line_num)
         if (token_line[pos].get_type() == ".double" and
-                token_line[pos].get_val() > float(2 * 10 ** 14)):
-            raise TooBigForDouble(str(token_line[pos].get_val()))
-        return token_line[pos].get_val(), pos + 1
+                token_line[pos].get_val(line_num) > float(2 * 10 ** 14)):
+            raise TooBigForDouble(str(token_line[pos].get_val(line_num)),
+                                  line_num)
+        return token_line[pos].get_val(line_num), pos + 1
     elif isinstance(token_line[pos], QuestionTok):
         return DONT_INIT, pos + 1
     else:
-        raise InvalidArgument(token_line[pos].get_nm())
+        raise InvalidArgument(token_line[pos].get_nm(), line_num)
 
 
-def get_DUP_value(token_line, pos):
+def get_DUP_value(token_line, pos, line_num):
     """
     Finds the value to duplicate if found
 
@@ -239,19 +245,19 @@ def get_DUP_value(token_line, pos):
     if isinstance(token_line[pos], OpenParen):
         pos += 1
     else:
-        raise MissingOpenParen()
-    dup_value, pos = get_data_token(token_line, pos)
+        raise MissingOpenParen(line_num)
+    dup_value, pos = get_data_token(token_line, pos, line_num)
     if isinstance(token_line[pos], CloseParen):
         return dup_value, pos + 1
     else:
-        raise MissingCloseParen()
+        raise MissingCloseParen(line_num)
 
 
-def is_str_termin(token_line, pos):
+def is_str_termin(token_line, pos, line_num):
     if not isinstance(token_line[pos], IntegerTok):
         return False
     # check if following is an Integer with value of 0
-    elif (token_line[pos].get_val() != 0):
+    elif (token_line[pos].get_val(line_num) != 0):
         return False
     return True
 
@@ -261,7 +267,7 @@ COMMA_POS = 1
 TERM_POS = 2
 
 
-def parse_string_token(token_line, pos):
+def parse_string_token(token_line, pos, line_num):
     """
     Creates a list of the string token's ASCII values
 
@@ -273,12 +279,13 @@ def parse_string_token(token_line, pos):
         List of ASCII values, followed by a 0
     """
     if len(token_line) < pos + NUM_STR_TOKS:
-        raise MissingData()
+        raise MissingData(line_num)
     elif not isinstance(token_line[pos + COMMA_POS], Comma):
-        raise MissingComma()
+        raise MissingComma(line_num)
 # is string terminated properly?
-    elif not is_str_termin(token_line, pos + TERM_POS):
-        raise InvalidDataVal(str(token_line[pos + TERM_POS].get_val()))
+    elif not is_str_termin(token_line, pos + TERM_POS, line_num):
+        raise InvalidDataVal(str(token_line[pos + TERM_POS].get_val(line_num)),
+                             line_num)
 
     else:
         ascii_list = []
@@ -289,7 +296,7 @@ def parse_string_token(token_line, pos):
         return (ascii_list, pos + 3)
 
 
-def parse_dup_token(token_line, data_type, pos):
+def parse_dup_token(token_line, data_type, pos, line_num):
     """
     Parses integer token followed by DUP token
 
@@ -302,8 +309,8 @@ def parse_dup_token(token_line, data_type, pos):
         List of duplicated values
     """
     dup_list = None
-    duplicate = token_line[pos].get_val()
-    value, pos = get_DUP_value(token_line, pos + 2)
+    duplicate = token_line[pos].get_val(line_num)
+    value, pos = get_DUP_value(token_line, pos + 2, line_num)
     if value == DONT_INIT:
         dup_list = [randrange(0, dtype_info[data_type][MAX_VAL])] * duplicate
     else:
@@ -311,7 +318,7 @@ def parse_dup_token(token_line, data_type, pos):
     return (dup_list, pos)
 
 
-def get_values(token_line, data_type, pos, values_list):
+def get_values(token_line, data_type, pos, values_list, line_num):
     """
     Creates a list of values for each variable when it is declared.
 
@@ -325,12 +332,12 @@ def get_values(token_line, data_type, pos, values_list):
         List of integer values
     """
     if pos >= len(token_line):
-        raise MissingData()
+        raise MissingData(line_num)
     elif isinstance(token_line[pos], StringTok):
-        ascii_list, pos = parse_string_token(token_line, pos)
+        ascii_list, pos = parse_string_token(token_line, pos, line_num)
         values_list.extend(ascii_list)
     else:
-        first_data, pos = get_data_token(token_line, pos)
+        first_data, pos = get_data_token(token_line, pos, line_num)
         if first_data == DONT_INIT:
             values_list.append(randrange(0, dtype_info[data_type][MAX_VAL]))
         else:
@@ -341,19 +348,21 @@ def get_values(token_line, data_type, pos, values_list):
         next_term = token_line[pos]
         if isinstance(next_term, DupTok):
             values_list.pop()
-            dup_list, pos = parse_dup_token(token_line, data_type, pos - 1)
+            dup_list, pos = parse_dup_token(token_line, data_type,
+                                            pos - 1, line_num)
             values_list.extend(dup_list)
             try:
                 next_term = token_line[pos]
             except Exception:
                 return values_list, pos
         if isinstance(next_term, Comma):
-            return get_values(token_line, data_type, pos + 1, values_list)
+            return get_values(token_line, data_type, pos + 1,
+                              values_list, line_num)
         else:
-            raise InvalidDataVal(token_line[pos].get_nm())
+            raise InvalidDataVal(token_line[pos].get_nm(), line_num)
 
 
-def parse_data_token(token_line, vm, mem_loc):
+def parse_data_token(token_line, vm, mem_loc, line_num):
     """
     Parses data tokens, assigns each value to a memory location
 
@@ -369,12 +378,13 @@ def parse_data_token(token_line, vm, mem_loc):
     symbol = ""
     data_vals = []
     if not isinstance(token_line[pos], NewSymbol):
-        raise InvalidArgument(token_line[pos].get_nm())
+        raise InvalidArgument(token_line[pos].get_nm(), line_num)
     else:
         symbol = token_line[pos].get_nm()
-    data_type = get_data_type(token_line, pos + 1)
+    data_type = get_data_type(token_line, pos + 1, line_num)
     pos += 2
-    data_vals, pos = get_values(token_line, data_type, pos, data_vals)
+    data_vals, pos = get_values(token_line, data_type, pos,
+                                data_vals, line_num)
 
     # store memory location
     vm.symbols[symbol] = mem_loc
@@ -389,7 +399,7 @@ def parse_data_token(token_line, vm, mem_loc):
     return mem_loc
 
 
-def get_term(token_line, pos, vm):
+def get_term(token_line, pos, vm, line_num):
     """
     Returns the next term of the expression
 
@@ -405,9 +415,9 @@ def get_term(token_line, pos, vm):
     if isinstance(token_line[pos], MinusTok):
         try:
             token_line[pos + 1].negate_val()
-            return get_term(token_line, pos + 1, vm)
+            return get_term(token_line, pos + 1, vm, line_num)
         except Exception:
-            raise InvalidArgument(token_line[pos].get_nm())
+            raise InvalidArgument(token_line[pos].get_nm(), line_num)
     # integer or register term
     elif (isinstance(token_line[pos], IntegerTok) or
           isinstance(token_line[pos], Register)):
@@ -417,9 +427,9 @@ def get_term(token_line, pos, vm):
         if token_line[pos].get_nm() in vm.symbols:
             return (Symbol(token_line[pos].get_nm(), vm), pos)
         else:
-            raise InvalidMemLoc(token_line[pos].get_nm())
+            raise InvalidMemLoc(token_line[pos].get_nm(), line_num)
     else:
-        raise InvalidMemLoc(token_line[pos].get_nm())
+        raise InvalidMemLoc(token_line[pos].get_nm(), line_num)
 
 
 REG = 0
@@ -427,7 +437,7 @@ DISP_VAL = 1
 POSITION = 2
 
 
-def get_expr_intel(token_line, pos, vm, reg):
+def get_expr_intel(token_line, pos, vm, line_num, reg):
     """
     Returns the register and the evaluated expression
 
@@ -440,27 +450,29 @@ def get_expr_intel(token_line, pos, vm, reg):
         Register token, displacement, next position
     """
     if len(token_line) < pos + 2:
-        return MissingOps()
-    left, pos = get_term(token_line, pos, vm)
+        return MissingOps(line_num)
+    left, pos = get_term(token_line, pos, vm, line_num)
     if isinstance(left, Register):
         reg = left
     next_term = token_line[pos + 1]
     if isinstance(next_term, PlusTok):
-        next_val_pos = get_expr_intel(token_line, pos + 2, vm, reg)
-        return (next_val_pos[REG], left.get_val() + next_val_pos[DISP_VAL],
+        next_val_pos = get_expr_intel(token_line, pos + 2, vm, line_num, reg)
+        return (next_val_pos[REG],
+                left.get_val(line_num) + next_val_pos[DISP_VAL],
                 next_val_pos[POSITION])
     elif isinstance(next_term, MinusTok):
-        next_val_pos = get_expr_intel(token_line, pos + 2, vm, reg)
-        return (next_val_pos[REG], left.get_val() - next_val_pos[DISP_VAL],
+        next_val_pos = get_expr_intel(token_line, pos + 2, vm, line_num, reg)
+        return (next_val_pos[REG],
+                left.get_val(line_num) - next_val_pos[DISP_VAL],
                 next_val_pos[POSITION])
     else:
-        return (reg, left.get_val(), pos + 1)
+        return (reg, left.get_val(line_num), pos + 1)
 
 
 SEC_REG = 0
 
 
-def get_expr_att(token_line, pos, vm, reg, disp_list):
+def get_expr_att(token_line, pos, vm, line_num, reg, disp_list):
     """
     Returns address expression for AT&T
 
@@ -474,8 +486,8 @@ def get_expr_att(token_line, pos, vm, reg, disp_list):
         Register token, displacement(s), next position
     """
     if len(token_line) < pos + 2:
-        return MissingOps()
-    left, pos = get_term(token_line, pos, vm)
+        return MissingOps(line_num)
+    left, pos = get_term(token_line, pos, vm, line_num)
 
     # Retrieved Register Term
     if isinstance(left, Register) and reg is None:
@@ -485,28 +497,28 @@ def get_expr_att(token_line, pos, vm, reg, disp_list):
 
     # at most two registers allowed
     elif isinstance(left, Register):
-        raise InvalidMemLoc(token_line[pos].get_nm())
+        raise InvalidMemLoc(token_line[pos].get_nm(), line_num)
 
     # Retrieved Integer term
     elif isinstance(left, IntegerTok):
         if token_line[pos - 2] == reg:
-            reg.set_multiplier(left.get_val())
+            reg.set_multiplier(left.get_val(line_num))
         elif token_line[pos - 2] == disp_list[SEC_REG]:
-            disp_list[SEC_REG].set_multiplier(left.get_val())
+            disp_list[SEC_REG].set_multiplier(left.get_val(line_num))
         else:
-            disp_list.append(left.get_val())
+            disp_list.append(left.get_val(line_num))
 
     # Retrieved Symbol term
     else:
         disp_list.append(vm.symbols[token_line[pos].get_nm()])
     next_term = token_line[pos + 1]
     if isinstance(next_term, Comma):
-        return get_expr_att(token_line, pos + 2, vm, reg, disp_list)
+        return get_expr_att(token_line, pos + 2, vm, line_num, reg, disp_list)
     else:
         return (reg, disp_list, pos + 1)
 
 
-def get_expr_mips(token_line, pos, vm):
+def get_expr_mips(token_line, pos, vm, line_num):
     """
     Returns address expression for MIPS
 
@@ -520,15 +532,15 @@ def get_expr_mips(token_line, pos, vm):
     """
 
     if pos >= len(token_line):
-        raise MissingOps()
-    left, pos = get_term(token_line, pos, vm)
+        raise MissingOps(line_num)
+    left, pos = get_term(token_line, pos, vm, line_num)
     if isinstance(left, Register):
         return (left, pos + 1)
     else:
-        raise InvalidMemLoc(left.get_nm())
+        raise InvalidMemLoc(left.get_nm(), line_num)
 
 
-def get_address_intel(token_line, pos, vm):
+def get_address_intel(token_line, pos, vm, line_num):
     """
     Converts a sublist of the tokenized instruction into
     corresponding address token
@@ -543,19 +555,19 @@ def get_address_intel(token_line, pos, vm):
     """
 
     if pos >= len(token_line):
-        raise InvalidMemLoc("")
+        raise InvalidMemLoc("", line_num)
     reg = None
     disp = 0
-    reg, disp, pos = get_expr_intel(token_line, pos, vm, reg)
+    reg, disp, pos = get_expr_intel(token_line, pos, vm, line_num, reg)
     if pos >= len(token_line):
-        raise MissingCloseBrack()
+        raise MissingCloseBrack(line_num)
     elif isinstance(token_line[pos], CloseBracket):
         return (reg, disp, pos + 1)
     else:
-        raise InvalidMemLoc(token_line[pos].get_nm())
+        raise InvalidMemLoc(token_line[pos].get_nm(), line_num)
 
 
-def get_address_att(token_line, pos, vm, disp=0):
+def get_address_att(token_line, pos, vm, line_num, disp=0):
     """
     Converts a sublist of the tokenized instruction into
     corresponding address token for AT&T
@@ -570,11 +582,12 @@ def get_address_att(token_line, pos, vm, disp=0):
         Register token, displacement(s), next position
     """
     if pos >= len(token_line):
-        raise InvalidMemLoc("")
+        raise InvalidMemLoc("", line_num)
     reg = None
-    reg, disp_list, pos = get_expr_att(token_line, pos, vm, reg, [None])
+    reg, disp_list, pos = get_expr_att(token_line, pos, vm,
+                                       line_num, reg, [None])
     if pos >= len(token_line):
-        raise MissingCloseParen()
+        raise MissingCloseParen(line_num)
     elif isinstance(token_line[pos], CloseParen):
         if len(disp_list) > 1:
             for values in range(1, len(disp_list)):
@@ -583,10 +596,10 @@ def get_address_att(token_line, pos, vm, disp=0):
             return (reg, disp, pos + 1)
         return (reg, [disp_list[0], disp], pos + 1)
     else:
-        raise InvalidMemLoc(token_line[pos].get_nm())
+        raise InvalidMemLoc(token_line[pos].get_nm(), line_num)
 
 
-def get_address_mips(token_line, pos, vm, disp=0):
+def get_address_mips(token_line, pos, vm, line_num, disp=0):
     """
     Converts a sublist of the tokenized instruction into
     corresponding address token for MIPS
@@ -601,17 +614,17 @@ def get_address_mips(token_line, pos, vm, disp=0):
     """
 
     if pos >= len(token_line):
-        raise InvalidMemLoc("")
-    reg, pos = get_expr_mips(token_line, pos, vm)
+        raise InvalidMemLoc("", line_num)
+    reg, pos = get_expr_mips(token_line, pos, vm, line_num)
     if pos >= len(token_line):
-        raise MissingCloseParen()
+        raise MissingCloseParen(line_num)
     elif isinstance(token_line[pos], CloseParen):
         return (reg, disp, pos + 1)
     else:
-        raise InvalidMemLoc(token_line[pos].get_nm())
+        raise InvalidMemLoc(token_line[pos].get_nm(), line_num)
 
 
-def get_address_location(token_line, pos, vm):
+def get_address_location(token_line, pos, vm, line_num):
     """
     Retrieves address at current position in code
     Retrieves address by coding language
@@ -628,22 +641,22 @@ def get_address_location(token_line, pos, vm):
     reg = None
     disp = 0
     if vm.flavor == "intel":
-        reg, disp, pos = get_address_intel(token_line, pos, vm)
+        reg, disp, pos = get_address_intel(token_line, pos, vm, line_num)
     elif vm.flavor == "att":
-        reg, disp, pos = get_address_att(token_line, pos, vm)
+        reg, disp, pos = get_address_att(token_line, pos, vm, line_num)
     else:
-        reg, disp, pos = get_address_mips(token_line, pos, vm)
+        reg, disp, pos = get_address_mips(token_line, pos, vm, line_num)
     if reg:
         return (RegAddress(reg.get_nm(), vm,
                            disp, reg.get_multiplier()), pos)
     else:
         # eliminates negative memory locations
         if disp < 0:
-            raise InvalidMemLoc(str(disp))
+            raise InvalidMemLoc(str(disp), line_num)
         return (Address(hex(disp).split('x')[-1].upper(), vm), pos)
 
 
-def check_constant(token_line, pos):
+def check_constant(token_line, pos, line_num):
     """
     Checks if the following token after constant token is valid
 
@@ -664,13 +677,13 @@ def check_constant(token_line, pos):
             try:
                 token_line[pos + 2].con = True
             except Exception:
-                raise InvalidArgument("-")
+                raise InvalidArgument("-", line_num)
         return True
     except Exception:
-        raise InvalidArgument("$")
+        raise InvalidArgument("$", line_num)
 
 
-def get_op(token_line, pos, vm):
+def get_op(token_line, pos, vm, line_num):
     """
     Retrieves operand of instruction
 
@@ -683,11 +696,11 @@ def get_op(token_line, pos, vm):
         Operand token, position of next item in instruction
     """
     if pos >= len(token_line):
-        raise MissingOps()
+        raise MissingOps(line_num)
 
 # Register
     elif isinstance(token_line[pos], Register):
-        return register_token(token_line, pos, vm)
+        return register_token(token_line, pos, vm, line_num)
 
 # Floating Point Token
     elif isinstance(token_line[pos], FloatTok):
@@ -695,19 +708,19 @@ def get_op(token_line, pos, vm):
 
 # Constant Token
     elif isinstance(token_line[pos], ConstantSign):
-        if vm.flavor == "att" and check_constant(token_line, pos):
-            return get_op(token_line, pos + 1, vm)
+        if vm.flavor == "att" and check_constant(token_line, pos, line_num):
+            return get_op(token_line, pos + 1, vm, line_num)
         else:
-            raise InvalidArgument("$")
+            raise InvalidArgument("$", line_num)
 
 # Minus Token
     elif isinstance(token_line[pos], MinusTok):
-        minus_token(token_line, pos)
-        return get_op(token_line, pos + 1, vm)
+        minus_token(token_line, pos, line_num)
+        return get_op(token_line, pos + 1, vm, line_num)
 
 # Integer Token
     elif isinstance(token_line[pos], IntegerTok):
-        return number_token(token_line, pos, vm)
+        return number_token(token_line, pos, vm, line_num)
 
 # Symbol/Label Token
     elif isinstance(token_line[pos], NewSymbol):
@@ -717,20 +730,20 @@ def get_op(token_line, pos, vm):
             if token_line[pos].get_nm() in vm.labels:
                 return (Label(token_line[pos].get_nm(), vm), pos + 1)
             elif token_line[pos].get_nm() in vm.symbols:
-                return symbol_token(token_line, pos, vm)
+                return symbol_token(token_line, pos, vm, line_num)
             elif vm.flavor == 'intel' and token_line[pos].get_nm()[:2] == "ST":
                 return token_line[pos], pos + 1
             else:
-                raise UnknownName(token_line[pos].get_nm())
+                raise UnknownName(token_line[pos].get_nm(), line_num)
 
 # Address Token
     elif is_start_address(token_line, pos, vm.flavor):
-        return get_address_location(token_line, pos + 1, vm)
+        return get_address_location(token_line, pos + 1, vm, line_num)
     else:
-        raise InvalidArgument(token_line[pos].get_nm())
+        raise InvalidArgument(token_line[pos].get_nm(), line_num)
 
 
-def get_op_list(token_line, pos, vm, op_lst):
+def get_op_list(token_line, pos, vm, op_lst, line_num):
     """
     Returns a list of ops
 
@@ -743,20 +756,19 @@ def get_op_list(token_line, pos, vm, op_lst):
     Returns:
         A list of ops, next position
     """
-    op, pos = get_op(token_line, pos, vm)
+    op, pos = get_op(token_line, pos, vm, line_num)
     op_lst.append(op)
     if pos >= len(token_line):
         return op_lst, pos
     else:
-
         next_op = token_line[pos]
         if isinstance(next_op, Comma):
-            return get_op_list(token_line, pos + 1, vm, op_lst)
+            return get_op_list(token_line, pos + 1, vm, op_lst, line_num)
         else:
-            raise MissingComma()
+            raise MissingComma(line_num)
 
 
-def get_pc(token_line, pos):
+def get_pc(token_line, pos, line_num):
     """
     Returns the PC counter of the instruction
 
@@ -768,14 +780,14 @@ def get_pc(token_line, pos):
         Integer token of PC counter value
     """
     if not isinstance(token_line[pos], IntegerTok):
-        raise MissingPc()
-    elif token_line[pos].get_val() % 4 != 0:
-        raise InvalidPc(str(token_line[pos].get_val()))
+        raise MissingPc(line_num)
+    elif token_line[pos].get_val(line_num) % 4 != 0:
+        raise InvalidPc(str(token_line[pos].get_val(line_num)), line_num)
     else:
         return token_line[pos]
 
 
-def parse_exec_unit(token_line, vm):
+def parse_exec_unit(token_line, vm, line_num):
     """
     Parses instruction
 
@@ -793,18 +805,18 @@ def parse_exec_unit(token_line, vm):
     # retrieve PC counter
     if (vm.flavor == "mips_asm" or vm.flavor == "mips_mml" or
             vm.flavor == "riscv"):
-        token_instruction.append(get_pc(token_line, pos))
+        token_instruction.append(get_pc(token_line, pos, line_num))
         pos += 1
 
     # retrieve instruction
     if not isinstance(token_line[pos], Instruction):
-        raise InvalidInstruction(token_line[pos].get_nm())
+        raise InvalidInstruction(token_line[pos].get_nm(), line_num)
     token_instruction.append(token_line[pos])
     pos += 1
 
     # retrieve ops
     if pos < len(token_line):
-        op_lst, pos = get_op_list(token_line, pos, vm, op_lst)
+        op_lst, pos = get_op_list(token_line, pos, vm, op_lst, line_num)
     token_instruction.extend(op_lst)
     # switch ops if flavor is AT&T
     if vm.flavor == 'att' and len(token_instruction) > 2:
@@ -815,13 +827,15 @@ def parse_exec_unit(token_line, vm):
     return token_instruction
 
 
-def parse(tok_lines, vm):
+def parse(tok_lines, vm, web):
     """
     Parses the analysis obtained from lexical analysis
 
     Args:
         tok_lines: Lines containing each line of code
         vm: Virtual machine
+        web: Boolean indicating whether source is from the website
+             or from kernel
 
     Returns:
         A list of parsed instructions
@@ -831,29 +845,32 @@ def parse(tok_lines, vm):
     token_instrs = []
     mem_loc = 0
     ip_init = None
-    for tokens in tok_lines:
-
-        if isinstance(tokens[0][TOKENS], Section):
-            if tokens[0][TOKENS].get_nm() == "data":
+    for line in tok_lines:
+        tokens = line[TOKENS]
+        line_num = line[LINE_NUMBER]
+        if isinstance(tokens[0], Section):
+            if tokens[0].get_nm() == "data":
                 parse_data = True
                 parse_text = False
+                if not web:
+                    vm.set_data_init("on")
+                    vm.re_init()
                 continue
-            elif tokens[0][TOKENS].get_nm() == "text":
+            elif tokens[0].get_nm() == "text":
                 parse_text = True
                 parse_data = False
                 continue
             else:
-                raise InvalidSection(tokens[0][TOKENS].get_nm())
+                raise InvalidSection(tokens[0].get_nm(), line_num)
         if parse_data:
-            vm.set_data_init("on")
-            mem_loc = parse_data_token(tokens[0], vm, mem_loc)
+            mem_loc = parse_data_token(tokens, vm, mem_loc, line_num)
         elif parse_text:
             vm.set_data_init("off")
-            parsed_unit = parse_exec_unit(tokens[0], vm)
-            token_instrs.append((parsed_unit, tokens[1]))
+            parsed_unit = parse_exec_unit(tokens, vm, line_num)
+            token_instrs.append((parsed_unit, line[CODE], line_num))
             if (vm.flavor == "mips_asm" or
                 vm.flavor == "mips_mml" or
                     vm.flavor == "riscv") and ip_init is None:
-                ip_init = token_instrs[0][TOKENS][0].get_val()
+                ip_init = token_instrs[0][TOKENS][0].get_val(line_num)
                 vm.start_ip = ip_init
     return token_instrs
