@@ -4,7 +4,9 @@ Our x86 virtual machine representation.
 
 from collections import OrderedDict
 
-from .errors import StackOverflow, StackUnderflow
+from .errors import StackOverflow, StackUnderflow, InvalidArgument
+from .MIPS.control_flow import Jal, Jr
+from .MIPS.key_words import op_func_codes
 
 MEM_DIGITS = 2
 
@@ -22,6 +24,10 @@ STACK_PTR_INTEL = "ESP"
 STACK_PTR_MIPS = "R29"
 STACK_PTR_RISCV = "X2"
 
+# Why are these the right values?
+MIPS_INTERRUPT_IP = 2147484032
+RISC_INTERRUPT_IP = 470351872
+
 
 class VirtualMachine:
     """
@@ -52,6 +58,7 @@ class VirtualMachine:
         self.flavor = None
         self.data_init = "on"
         self.start_ip = 0
+        self.ip_div = 1
         self.changes = set()
         self.base = None
         self.stack_change = ""
@@ -64,6 +71,24 @@ class VirtualMachine:
                 + "Stack: " + str(self.stack) + "\n"
                 + "Labels: " + str(self.labels))
 
+    def get_ip_div(self):
+        return self.ip_div
+
+    def get_start_ip(self):
+        return self.start_ip
+
+    def get_ip(self):
+        pass
+
+    def set_ip(self):
+        pass
+
+    def past_last_instr(self, token_lines):
+        curr_ip = self.get_ip() - self.start_ip
+        if curr_ip // self.get_ip_div() >= len(token_lines):
+            return True
+        return False
+
     def re_init(self):
         self.nxt_key = 0
         self.ip = 0
@@ -119,21 +144,41 @@ class VirtualMachine:
     def set_data_init(self, on_or_off):
         self.data_init = on_or_off
 
+    def jump_to_label(self, label, source, jal=False):
+        if label in self.labels:
+            ip = self.labels[label]
+            self.set_ip(ip + self.start_ip)
+            self.next_stack_change = label
+            return (True, source, "")
+        else:
+            try:
+                ip = int(label)
+                if jal:
+                    ip = ip >> 2
+                self.set_ip(ip)
+                return (True, source, "")
+            except Exception:
+                return (False, source, f"Invalid label: {label}")
+
+    def jump_handler(self, brk, source, instr_line):
+        pass
+
+    def exec_instr(self, instr_line):
+        pass
+
+    def create_bit_instr(self, instr_lst):
+        return ''
+
+    def set_next_stack_change(self):
+        for label in self.labels:
+            if self.get_ip() == self.labels[label]:
+                self.next_stack_change = label
+
 
 class IntelMachine(VirtualMachine):
     def __init__(self):
         super().__init__()
-        # self.fp_stack_registers = OrderedDict(
-        #             [
-        #                 ('ST7', 0.0),
-        #                 ('ST6', 0.0),
-        #                 ('ST5', 0.0),
-        #                 ('ST4', 0.0),
-        #                 ('ST3', 0.0),
-        #                 ('ST2', 0.0),
-        #                 ('ST1', 0.0),
-        #                 ('ST0', 0.0),
-        #             ])
+
         self.float_stack_bottom = -1
 
         self.registers = OrderedDict(
@@ -167,6 +212,8 @@ class IntelMachine(VirtualMachine):
                         ('SF', 0),
                         ('ZF', 0),
                     ])
+        self.instr = 0
+        self.ops = 1
 
     def is_FP_stack_empty(self):
         return self.float_stack_bottom == -1
@@ -224,34 +271,49 @@ class IntelMachine(VirtualMachine):
     def set_ip(self, val):
         self.registers[INSTR_PTR_INTEL] = val
 
+    def set_int_ip(self):
+        pass
+
     def get_ip(self):
         return int(self.registers[INSTR_PTR_INTEL])
 
-    def inc_sp(self):
+    def inc_sp(self, line_num):
         sp = self.get_sp()
         sp += 1
-        self.set_sp(sp)
+        self.set_sp(sp, line_num)
 
-    def dec_sp(self):
+    def dec_sp(self, line_num):
         sp = self.get_sp()
         sp -= 1
-        self.set_sp(sp)
+        self.set_sp(sp, line_num)
 
-    def set_sp(self, val):
+    def set_sp(self, val, line_num):
         if val < STACK_BOTTOM - 1:
-            raise StackOverflow()
+            raise StackOverflow(line_num)
         if val > STACK_TOP:
-            raise StackUnderflow()
+            raise StackUnderflow(line_num)
 
         self.registers[STACK_PTR_INTEL] = val
 
     def get_sp(self):
         return int(self.registers[STACK_PTR_INTEL])
 
+    def jump_handler(self, brk, source, instr_line=None):
+        return self.jump_to_label(brk.label, source, True)
+
+    def exec_instr(self, instr_line, line_num):
+        self.inc_ip()
+        return instr_line[self.instr].f(instr_line[self.ops:], self, line_num)
+
 
 class MIPSMachine(VirtualMachine):
+    """
+    Create a VM for running MIPS assembler.
+    Why isn't flavor set here?
+    """
     def __init__(self):
         super().__init__()
+        self.ip_div = 4
         self.unwritable = [INSTR_PTR_MIPS, 'R0', 'F0', 'F29',
                            STACK_PTR_MIPS, 'HI', 'LO']
         self.registers = OrderedDict(
@@ -330,6 +392,10 @@ class MIPSMachine(VirtualMachine):
                         ('COND', 0),
                     ])
 
+        self.pc = 0
+        self.instr = 1
+        self.ops = 2
+
     def re_init(self):
         super().re_init()
         self.registers[STACK_PTR_MIPS] = STACK_TOP
@@ -347,36 +413,234 @@ class MIPSMachine(VirtualMachine):
     def set_ip(self, val):
         self.registers[INSTR_PTR_MIPS] = val
 
+    def set_int_ip(self):
+        self.set_ip(MIPS_INTERRUPT_IP)
+
     def get_ip(self):
         return int(self.registers[INSTR_PTR_MIPS])
 
-    def inc_sp(self):
+    def inc_sp(self, line_num):
         sp = self.get_sp()
         sp += 1
-        self.set_sp(sp)
+        self.set_sp(sp, line_num)
 
-    def dec_sp(self):
+    def dec_sp(self, line_num):
         sp = self.get_sp()
         sp -= 1
-        self.set_sp(sp)
+        self.set_sp(sp, line_num)
 
-    def set_sp(self, val):
+    def set_sp(self, val, line_num):
         if val < STACK_BOTTOM - 1:
-            raise StackOverflow()
+            raise StackOverflow(line_num)
         if val > STACK_TOP:
-            raise StackUnderflow()
+            raise StackUnderflow(line_num)
 
         self.registers[STACK_PTR_MIPS] = val
 
     def get_sp(self):
         return int(self.registers[STACK_PTR_MIPS])
 
+    def create_bit_negative(self, value, bits):
+        """
+        Converts an immediate value into a string of bits
+
+        Args:
+            value: Immediat value
+            bits: Number of bits needed
+
+        Returns:
+            Formatted binary value of immediate
+            in the number of bits inputted
+        """
+        imm_code = bin(value).split('b')[1]
+        imm_code = '0'*(bits - len(imm_code)) + imm_code
+        if value < 0:
+            imm_lst = []
+            for bit in imm_code:
+                imm_lst.append(bit)
+            flip_bit = False
+            place = bits - 1
+            while place >= 0:
+                if not flip_bit and imm_lst[place] == "1":
+                    flip_bit = True
+                elif flip_bit:
+                    if imm_lst[place] == "0":
+                        imm_lst[place] = "1"
+                    else:
+                        imm_lst[place] = "0"
+                place -= 1
+            imm_code = "".join(imm_lst)
+        return imm_code
+
+    def create_bit_r_format(self, instr_lst, op_func, func_code):
+        """
+        Converts an R-format instruction into a string of bits
+
+        Args:
+            instr_lst: Line of code
+
+        Returns:
+            Formatted version of instruction
+        """
+        rs = 0
+        rt = 0
+        shamt = 0
+        rd = 0
+
+        # if instruction has shift value
+        if func_code == "000000" or func_code == "000010":
+            try:
+                rd = int(instr_lst[self.ops].get_nm().split('R')[1])
+                rt = int(instr_lst[self.ops + 1].get_nm().split('R')[1])
+                shamt = instr_lst[self.ops + 2].get_val()
+            except Exception:
+                pass
+
+        # if function does not use rd
+        elif func_code == "011000" or func_code == "011010":
+            try:
+                rs = int(instr_lst[self.ops].get_nm().split('R')[1])
+                rt = int(instr_lst[self.ops + 1].get_nm().split('R')[1])
+            except Exception:
+                pass
+
+        # if function only uses rd
+        elif func_code == "010010" or func_code == "010000":
+            try:
+                rd = int(instr_lst[self.ops].get_nm().split('R')[1])
+            except Exception:
+                pass
+
+        # if function only uses rs
+        elif func_code == "001000":
+            try:
+                rs = int(instr_lst[self.ops].get_nm().split('R')[1])
+            except Exception:
+                pass
+
+        # other arithmetic, logic functions
+        else:
+            try:
+                rs = int(instr_lst[self.ops + 2].get_nm().split('R')[1])
+                rt = int(instr_lst[self.ops + 1].get_nm().split('R')[1])
+                rd = int(instr_lst[self.ops].get_nm().split('R')[1])
+            except Exception:
+                pass
+        # format the rs, rt, rd, shamt values into 5 bits
+        rs = format(rs, '#07b').split('b')[1]
+        rt = format(rt, '#07b').split('b')[1]
+        rd = format(rd, '#07b').split('b')[1]
+        shamt = format(shamt, '#07b').split('b')[1]
+        code_lst = [op_func, rs, rt, rd, shamt, func_code, "\n"]
+        return " ".join(code_lst)
+
+    def create_bit_i_format(self, instr_lst, op_func):
+        """
+        Converts an I-format instruction into a string of bits
+
+        Args:
+            instr_lst: Line of code
+
+        Returns:
+            Formatted version of instruction
+        """
+        imm = 0
+        rs = 0
+        rt = 0
+        # if not data movement instruction
+        if op_func != "100011" and op_func != "101011":
+            try:
+                rs = int(instr_lst[self.ops + 1].get_nm().split('R')[1])
+                rt = int(instr_lst[self.ops].get_nm().split('R')[1])
+                imm = int(instr_lst[self.ops + 2].get_val())
+            except Exception:
+                pass
+
+        # if LW or SW
+        else:
+            try:
+                rs = int(instr_lst[self.ops + 1].get_nm().split('R')[1])
+                rt = int(instr_lst[self.ops].get_nm().split('R')[1])
+                imm = int(instr_lst[self.ops + 1].displacement)
+            except Exception:
+                pass
+
+        # format rs, rt into 5 bits
+        rs = format(rs, '#07b').split('b')[1]
+        rt = format(rt, '#07b').split('b')[1]
+
+        # format imm to 16 bits signed
+        imm_code = self.create_bit_negative(imm, 16)
+        code_lst = [op_func, rs, rt, imm_code, "\n"]
+        return " ".join(code_lst)
+
+    def create_bit_j_format(self, instr_lst, op_func):
+        """
+        Converts an J-format instruction into a string of bits
+
+        Args:
+            instr_lst: Line of code
+
+        Returns:
+            Formatted version of instruction
+        """
+        imm = 0
+        try:
+            imm = int(instr_lst[self.ops].get_val())
+        except Exception:
+            pass
+
+        # format imm into 26 bits signed
+        imm_code = self.create_bit_negative(imm, 26)
+        code_lst = [op_func, imm_code, "\n"]
+        return " ".join(code_lst)
+
+    def create_bit_instr(self, instr_lst):
+        """
+        Converts the instruction into code of bits
+
+        Args:
+            instr_lst: Line of code
+
+        Returns:
+            Formatted version of the instruction in bits
+        """
+        op_func = None
+        func_code = None
+        instr_nm = instr_lst[self.instr].get_nm()
+        try:
+            op_func, func_code = op_func_codes[instr_nm]
+        except Exception:
+            op_func = op_func_codes[instr_lst[self.instr].get_nm()]
+        if func_code is not None:
+            return self.create_bit_r_format(instr_lst, op_func, func_code)
+        else:
+            if instr_nm != "JAL" and instr_nm != "J":
+                return self.create_bit_i_format(instr_lst, op_func)
+            else:
+                return self.create_bit_j_format(instr_lst, op_func)
+
+    def jump_handler(self, brk, source, instr_line):
+        if isinstance(instr_line[self.instr], Jal):
+            self.registers["R31"] = self.get_ip()
+        if isinstance(instr_line[self.instr], Jr):
+            return self.jump_to_label(brk.label, source)
+        return self.jump_to_label(brk.label, source, True)
+
+    def exec_instr(self, instr_line, line_num):
+        if self.get_ip() != instr_line[self.pc].get_val(line_num):
+            raise InvalidArgument(hex(instr_line[self.pc].get_val(line_num)),
+                                  line_num)
+        self.inc_ip()
+        return instr_line[self.instr].f(instr_line[self.ops:], self, line_num)
+
 
 class RISCVMachine(VirtualMachine):
     # make sure to account for the lack of HI and LO in display
 
     def __init__(self):
         super().__init__()
+        self.ip_div = 4
         self.unwritable = [INSTR_PTR_RISCV, 'X0', STACK_PTR_RISCV]
         self.registers = OrderedDict(
                         [
@@ -419,6 +683,10 @@ class RISCVMachine(VirtualMachine):
                         ('COND', 0),
                     ])
 
+        self.pc = 0
+        self.instr = 1
+        self.ops = 2
+
     def re_init(self):
         super().re_init()
         self.registers[STACK_PTR_RISCV] = STACK_TOP
@@ -436,30 +704,43 @@ class RISCVMachine(VirtualMachine):
     def set_ip(self, val):
         self.registers[INSTR_PTR_RISCV] = val
 
+    def set_int_ip(self):
+        self.set_ip(RISC_INTERRUPT_IP)
+
     def get_ip(self):
         return int(self.registers[INSTR_PTR_RISCV])
 
-    def inc_sp(self):
+    def inc_sp(self, line_num):
         sp = self.get_sp()
         sp += 1
-        self.set_sp(sp)
+        self.set_sp(sp, line_num)
 
-    def dec_sp(self):
+    def dec_sp(self, line_num):
         sp = self.get_sp()
         sp -= 1
-        self.set_sp(sp)
+        self.set_sp(sp, line_num)
 
-    def set_sp(self, val):
+    def set_sp(self, val, line_num):
         if val < STACK_BOTTOM - 1:
-            raise StackOverflow()
+            raise StackOverflow(line_num)
         if val > STACK_TOP:
-            raise StackUnderflow()
+            raise StackUnderflow(line_num)
 
         self.registers[STACK_PTR_RISCV] = val
 
     def get_sp(self):
         return int(self.registers[STACK_PTR_RISCV])
 
+    def exec_instr(self, instr_line, line_num):
+        if self.get_ip() != instr_line[self.pc].get_val(line_num):
+            raise InvalidArgument(hex(instr_line[self.pc].get_val(line_num)),
+                                  line_num)
+        self.inc_ip()
+        return instr_line[self.instr].f(instr_line[self.ops:], self, line_num)
+
+    def jump_handler(self, brk, source, instr_line):
+        return self.jump_to_label(brk.label, source, True)
+
 
 class WASMMachine(VirtualMachine):
     def __init__(self):
@@ -473,6 +754,9 @@ class WASMMachine(VirtualMachine):
         self.stack_ptr = STACK_BOTTOM
         self.ip = 0
 
+        self.instr = 0
+        self.ops = 1
+
     def re_init(self):
         self.nxt_key = 0
         self.ip = 0
@@ -499,21 +783,21 @@ class WASMMachine(VirtualMachine):
         for i in range(STACK_TOP - 3, STACK_BOTTOM - 1, -4):
             self.stack[hex(i).split('x')[-1].upper()] = 0
 
-    def inc_sp(self):
+    def inc_sp(self, line_num):
         sp = self.get_sp()
         sp += 4
-        self.set_sp(sp)
+        self.set_sp(sp, line_num)
 
-    def dec_sp(self):
+    def dec_sp(self, line_num):
         sp = self.get_sp()
         sp -= 4
-        self.set_sp(sp)
+        self.set_sp(sp, line_num)
 
-    def set_sp(self, val):
+    def set_sp(self, val, line_num):
         if val < STACK_BOTTOM - 1:
-            raise StackOverflow()
+            raise StackOverflow(line_num)
         if val > STACK_TOP:
-            raise StackUnderflow()
+            raise StackUnderflow(line_num)
 
         self.stack_ptr = val
 
@@ -531,6 +815,13 @@ class WASMMachine(VirtualMachine):
     def get_ip(self):
         return self.ip
 
+    def exec_instr(self, instr_line, line_num):
+        self.inc_ip()
+        return instr_line[self.instr].f(instr_line[self.ops:], self, line_num)
+
+    def set_next_stack_change(self):
+        pass
+
 
 intel_machine = IntelMachine()
 mips_machine = MIPSMachine()
